<?php
/* $Id: nc_core.class.php 3576 2009-11-15 15:55:13Z vadim $ */

class nc_Core extends nc_System {
  
  public $db;
  public $catalogue, $subdivision, $sub_class, $component, $template, $user;
  public $SYSTEM_FOLDER;
  public $inside_admin = false;
  public $admin_mode = false;
  public $is_trial = false; // trial version?
   
  protected function __construct () {
    global $SYSTEM_FOLDER;
    
    $this->set_variable("SYSTEM_FOLDER", $SYSTEM_FOLDER);
    
    // load parent constructor
    parent::__construct();
  }
  
  /**
   * Set object variable
   *
   * @param string variable name
   * @param mixed variable value
   */
  public function set_variable ($name, $value) {
    // set variable
    $this->$name = $value;
  }
  
  /**
   * Get object variable
   *
   * @param string variable name
   *
   * @return mixed variable value
   */
  public function get_variable ($name) {
    // return value
    return isset($this->$name) ? $this->$name : NULL;    
  }
  
  /**
   * Load system extension
   *
   * @param class name for loading 
   * @param mixed arguments for class __construct function
   *
   * @return instanteated object
   */
  public function load () {
    
    $args = func_get_args();
    
    $object = array_shift($args);
    
    if ( strstr($object, "/") ) {
      $object_arr = explode( "/", trim($object, "/") );
      $object = array_pop($object_arr);
      if ( !empty($object_arr) ) {
        $path = join("/", $object_arr)."/";
      }
    }
    
    if ( is_object($this->$object) ) {
      $this->debugMessage("System class \"".$object."\" already loaded", __FILE__, __LINE__, "info");
      return $this->$object;
    }
    
    $file_name = $this->SYSTEM_FOLDER.$path."nc_".$object.".class.php";
        
    if ( file_exists($file_name) ) {
      include_once ($file_name);
    }
    
    $class_name = "nc_".ucfirst($object);
    
    if ( class_exists($class_name) ) {
      $this->$object = call_user_func_array( array( new ReflectionClass($class_name), 'newInstance' ), $args );
    }
    
    if (!$this->$object) throw new Exception ("Unable load system class \"".$class_name."\"!");
    
    return $this->$object;
  }
  
  /**
   * Load system settings from base
   *
   * @return settings data array
   */
  public function get_settings ($item = "", $reset = false) {
    // call as static
    static $storage = array();
    
    // check inited object
    if ( empty($storage) || $reset ) {
      $storage = $this->db->get_row("SELECT settings.*, field.`Field_Name` AS UserEmail FROM `Settings` AS settings LEFT JOIN `Field` AS field ON settings.`UserEmailField` = field.`Field_ID` ORDER BY `Settings_ID` LIMIT 1", ARRAY_A);
      // send "powered by" header
      if (!$reset) Header("X-Powered-By: ".$storage['Powered']);
    }
    
    // if item requested return item value
    if ( $item && is_array($storage) ) {
      return array_key_exists($item, $storage) ? $storage[$item] : false;
    }
    
    // return all settings
    return $storage;
  }
  
  /**
   * Load default extensions
   *
   * @return settings data array
   */
  public function load_default_extensions () {
    // call as static
    static $loaded = false;
    
    // check inited object
    if (!$loaded) {
      // load default extensions
      $this->load("event");
      $this->load("db");
      $this->load("gzip");
      $this->load("input");
      $this->load("utf8");
      $this->load("url");
      $this->load("lang");
      $this->load("modules");
      // essences
      $this->load("essences/catalogue");
      $this->load("essences/subdivision");
      $this->load("essences/sub_class");
      $this->load("essences/component");
      $this->load("essences/user");
      $this->load("essences/template");
      $this->load("essences/message");
      $loaded = true;
    }
  }
  
  public function get_system_table_fields ($item = "") {
    // call as static
    static $storage = array();
    
     // check inited object
    if ( empty($storage) ) {
      // Load system table field info
      $res = $this->db->get_results("SELECT b.`System_Table_Name` AS system_table_name,
        a.`Field_ID` as id,
        a.`Field_Name` as name,
        a.`TypeOfData_ID` as type,
        a.`Inheritance` as inheritance,
        a.`Format` as format
        FROM `Field` AS a, `System_Table` AS b
        WHERE a.`System_Table_ID` = b.`System_Table_ID`", ARRAY_A);
        //ORDER BY a.`System_Table_ID`, a.`Priority`"
      // compile system table fields array 
      if ( !empty($res) ) {
        foreach ($res AS $row) {
          $storage[ $row['system_table_name'] ][] = $row;
        }
      }
    }
    
    // if item requested return item value
    if ( $item ) {
      return array_key_exists($item, $storage) ? $storage[$item] : array();
    }
    
    // return data associative array
    return $storage;
  }
  
  public function load_env() {
    global $admin_mode;
    global $catalogue, $sub, $cc;
    global $current_catalogue, $cc;
    global $current_sub;
    global $current_cc;
    global $cc_array;
    global $use_multi_sub_class;
    global $system_table_fields, $user_table_mode, $user_table_env;
    global $parent_sub_tree, $sub_level_count;
    
    $catalogue = $this->catalogue->get_current("Catalogue_ID");
    $sub = $this->subdivision->get_current("Subdivision_ID");
    $cc = $this->sub_class->get_current("Sub_Class_ID");

    $system_env = $this->get_settings();

    switch (true) {
      case !$catalogue:
        $catalogue = $this->db->get_var("SELECT `Catalogue_ID` FROM `Catalogue` ORDER BY `Priority` LIMIT 1");
        if ($catalogue) {
          $this->catalogue->set_current_by_id($catalogue);
        }
        else exit;

      case !$sub:
        $sub = $this->catalogue->get_by_id($catalogue, "Title_Sub_ID");
        if ($sub) {
          $this->subdivision->set_current_by_id($sub);
        }
        else exit;

      case !$cc:
        if (!$admin_mode) $checked_only = " AND `Checked` = 1";
        $cc = $this->db->get_var("SELECT `Sub_Class_ID` FROM `Sub_Class` WHERE `Subdivision_ID` = '".intval($sub)."'".$checked_only." ORDER BY `Priority` LIMIT 1");
        if ($cc) $this->sub_class->set_current_by_id($cc);
    }

    // Load all sub_class id's into array, may be exist in 
    if ( !is_array($cc_array) ) {
      $cc_array = array();
      // get cc(s) data
      $res = $this->sub_class->get_by_subdivision_id($sub);
      if ( !empty($res) ) {
        foreach ($res as $row) {
          if ($row['Checked']) $cc_array[] = $row['Sub_Class_ID'];
        }
      }
    }

    // load system table fields
    $system_table_fields = $this->get_system_table_fields();
    // set global variables
    $current_catalogue = $this->catalogue->get_current();
    $current_sub = $this->subdivision->get_current();
    $current_cc = $this->sub_class->get_current();

    if ( $current_cc['System_Table_ID']==3 || in_array($current_sub['Subdivision_ID'], preg_split("/\s*,\s*/", $this->modules->get_vars('auth', 'USER_MODIFY_SUB'))) ) {
      $action = "message";
      $user_table_mode = true;
      $user_table_env = LoadSystemTableClass("User");
    } 
    else {
      $user_table_mode = false;  
    }

    $parent_sub_tree[$sub_level_count]["Subdivision_Name"] = $current_catalogue["Catalogue_Name"];
    $parent_sub_tree[$sub_level_count]["Hidden_URL"] = "/";
        
    return;
  }
  
  /**
   * Get or instance self object
   *
   * @return self object
   */
  public static function get_object () {
    // call as static
    static $storage;
    // check inited object
    if ( !isset($storage) ) {
      // init object
      $storage = new self();
    }
    // return object
    return is_object($storage) ? $storage : false;
  }
  
}

?>